-----------------------------------------------------------------------------------------------
-- Client Lua Script for MovableFrames
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Apollo"
require "Window"
require "GameLib"
require "Spell"

-----------------------------------------------------------------------------------------------
-- MovableFrames Module Definition
-----------------------------------------------------------------------------------------------
local MovableFrames = {} 
local bAddonLoading = false
local tAddonList = {}
local tAccountList = nil
local nAddons = 28
local bReset = false
local bSave = false
local charName = nil
local realmName = nil
local nShiftM = nil
local questAlign = 2
for i=1,nAddons do
	tAddonList[i] = {}
end
local nVersion = 1.11
tAddonList[0] = {false}
tAddonList[1] = {"ActionBarFrame", "wndMain", "Action Bar"}
tAddonList[2] = {"ActionBarFrame", "wndBar2", "Bar Left"}
tAddonList[3] = {"ActionBarFrame", "wndBar3", "Bar Right"}
tAddonList[4] = {"CastBar", "wndOppFrame", "Castbar 2"}
tAddonList[5] = {"Datachron", "g_wndDatachron", "Datachron"}
tAddonList[6] = {"Datachron", "wndMinimized", "Datachron Toggle"}
tAddonList[7] = {"XPBar", "wndMain", "Experience Bar"}
tAddonList[8] = {"Hazards", "wndHazardListForm", "Hazard Menu"}
tAddonList[9] = {"InterfaceMenuList", "wndMain", "Interface Menu"}
tAddonList[10] = {"XPBar", "wndInvokeForm", "Inventory Bag"}
tAddonList[11] = {"LootNotificationWindow", "wndLootStack", "Loot Notification"}
tAddonList[12] = {"MiniMap", "wndMain", "MiniMap"}
tAddonList[13] = {"ActionBarFrame", "wndMountFlyout", "Mount Popout"}
tAddonList[14] = {"QuestTracker", "wndMain", "Quest Tracker"}
tAddonList[15] = {"RecallFrame", "wndMain", "Recall Frame"}
tAddonList[16] = {"Resource", "Resource", "Resource"}
tAddonList[17] = {"SprintMeter", "wndMain", "Sprint Meter"}
tAddonList[18] = {"ToolTips", "wndContainer", "ToolTips"}
tAddonList[19] = {"ActionBarFrame", "wndArt", "Window Art"}
tAddonList[20] = {"XPBar", "wndArt", "Window Art 2"}
tAddonList[21] = {"ZoneMap", "wndMain", "Zone Map"}
tAddonList[22] = {"TargetFrame", "luaFocusFrame", "- Focus Frame"}
tAddonList[23] = {"TargetFrame", "luaTargetFrame", "- Target Frame"}
tAddonList[24] = {"TargetFrame", "luaUnitFrame", "- Unit Frame"}
tAddonList[25] = {"HUDAlerts", "wndAlertContainer", "HUD Alerts"}
tAddonList[26] = {"RecallFrame", "wndMenu", "Recall Menu"}
tAddonList[27] = {"CastBar", "wndCastFrame", "Castbar"}
tAddonList[28] = {"HUDAlerts", "wndInteractForm", "HUD Interact"}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function MovableFrames:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tItems = {} -- keep track of all the list items
	o.MovableWindows = {}
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected
	o.ScaleWindows = {}
    return o
end

function MovableFrames:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureButton, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- MovableFrames OnLoad
-----------------------------------------------------------------------------------------------
function MovableFrames:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("MovableFrames.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.RegisterEventHandler("ActionBarReady", "OnActionBarReady", self)
end

-----------------------------------------------------------------------------------------------
-- MovableFrames OnDocLoaded
-----------------------------------------------------------------------------------------------
function MovableFrames:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "MovableFramesForm", "TooltipStratum", self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.wndBar2 = Apollo.LoadForm(self.xmlDoc, "Bar2ButtonContainer", "FixedHudStratum", self)
		self.wndBar3 = Apollo.LoadForm(self.xmlDoc, "Bar3ButtonContainer", "FixedHudStratum", self)

		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
	    self.wndMain:Show(false, true)
		Apollo.RegisterSlashCommand("mf", "OnMovableFramesOn", self)
		Apollo.RegisterSlashCommand("mfgrid", "OnGrid", self)
		Apollo.RegisterSlashCommand("mfreset", "OnReset", self)
		bAddonLoading = true
		self.timer = ApolloTimer.Create(1, true, "OnTimer", self)
	end
end

function MovableFrames:OnTimer()
	if bAddonLoading then
		self.timer = nil
		bAddonLoading = false
		charName = GameLib.GetPlayerUnit():GetName()
		realmName = GameLib.GetRealmName()
		self:GetClass()
		if tAddonList[0][4] then
			self.wndMain:SetAnchorPoints(unpack(tAddonList[0][4]))
			self.wndMain:SetAnchorOffsets(unpack(tAddonList[0][5]))
		end
		self:PopulateItemList()
		local cBox = self.wndMain:FindChild("ComboBox")
		if tAccountList then
			for k,v in pairs(tAccountList) do
				if tAccountList[k][0] then
					if tAccountList[k][0][2] then
						if tAccountList[k][0][2] < nVersion then
							--Don't Add
						else
							cBox:AddItem(k,"",k)
						end
					end
				end
			end
		end
		
		self:FrameScale()
		self:FramePositions(false)
		self:FrameHide()
		if tAddonList[0][6] == 0 then
			self.wndMain:FindChild("Sort"):SetCheck(true)
		end
	end	
end

function MovableFrames:OnActionBarReady()
	bAddonLoading = true
	self.timer = ApolloTimer.Create(1, true, "OnTimer", self)
end
-----------------------------------------------------------------------------------------------
-- MovableFrames Save and Restore
-----------------------------------------------------------------------------------------------
function MovableFrames:OnSave(eLevel)
	if eLevel == GameLib.CodeEnumAddonSaveLevel.General and charName ~= nil and realmName ~= nil and bSave then
			tAddonList[0][2] = nVersion
			tAddonList[0][4] = {self.wndMain:GetAnchorPoints()}
			tAddonList[0][5] = {self.wndMain:GetAnchorOffsets()}
		if tAccountList then
			self:FramePositions(true)
			tAccountList[charName .. "-" .. realmName] = tAddonList
			return tAccountList
		else
			self:FramePositions(true)
			tAccountList = {}
			tAccountList[charName .. "-" .. realmName] = tAddonList
			return tAccountList
		end
	end
	if eLevel == GameLib.CodeEnumAddonSaveLevel.Character then
		if not bReset then
			tAddonList[0][2] = nVersion
			tAddonList[0][4] = {self.wndMain:GetAnchorPoints()}
			tAddonList[0][5] = {self.wndMain:GetAnchorOffsets()}
			tAddonList[0][6] = questAlign
			self:FramePositions(true)
	    	return tAddonList
		else
			return {reset = true}
		end
	else
		return nil
    end
end

function MovableFrames:OnRestore(eLevel, tData)
	if tData then
		if eLevel == GameLib.CodeEnumAddonSaveLevel.General then
			if tData then
				tAccountList = tData
			end
		elseif eLevel == GameLib.CodeEnumAddonSaveLevel.Character and tData[0] then
			if tData then
				if not tData["reset"] then
					if tData[0] then
						if tData[0][2] then
							if tData[0][2] < 1.0 then
								-- do nothing tAccountList = nil
							else
								tAddonList = tData
								if tData[0][2] < 1.01 then
									tAddonList[0][4] = nil
									tAddonList[0][5] = nil
									tAddonList[0][6] = 2
								end
								questAlign = tAddonList[0][6]
								if tData[0][2] < 1.02 then
									tAddonList[28] = {"HUDAlerts", "wndInteractForm", "HUD Interact"}
								end
								if tData[0][2] < 1.09 then
									tAddonList[29] = {"CastBar", "wndOppFrame", "Castbar 2"}
									tAddonList[0][5][4] = tAddonList[0][5][4]+24
								end
								if tData[0][2] < 1.11 then
									tAddonList[4] = tAddonList[29]
									tAddonList[29] = nil
								end
							end			
						end
					end
				end
			end
		end
	end
end

function MovableFrames:OnReset(wndHandler, wndControl)
	bReset = true
	RequestReloadUI()
end

function MovableFrames:OnGrid(wndHandler, wndControl)
	self.wndGrid = Apollo.LoadForm(self.xmlDoc, "MfGrid", "InWorldHudStratum", self)
	local MySpace = 15
	local MfWidth = math.floor((self.wndGrid:GetWidth() / MySpace) / 2)
	local MfHeight = math.floor((self.wndGrid:GetHeight() / MySpace) / 2)
	local tMyGrid = {}
	tMyGrid[1] = {}
	tMyGrid[2] = {}
	for i=1, MfWidth do
		nR = i * MySpace
		nL = (i * MySpace) - 2
		tMyGrid[1][i]  = self.wndGrid:AddPixie({
  			strSprite = "BlackFill",
			loc = {
		    	fPoints = {0.5,0,0.5,1},
	    		nOffsets = {nL,0,nR,0}
			 }
		})

		tMyGrid[1][-i]  = self.wndGrid:AddPixie({
  			strSprite = "BlackFill",
			loc = {
		    	fPoints = {0.5,0,0.5,1},
	    		nOffsets = {-nR,0,-nL,0}
			 }
		})
	end
	for i=1, MfHeight do
		nB = i * MySpace
		nT = (i * MySpace) - 2
		tMyGrid[2][i]  = self.wndGrid:AddPixie({
  			strSprite = "BlackFill",
			loc = {
		    	fPoints = {0,0.5,1,0.5},
	    		nOffsets = {0,nT,0,nB}
			 }
		})
		tMyGrid[2][-i]  = self.wndGrid:AddPixie({
  			strSprite = "BlackFill",
			loc = {
		    	fPoints = {0,0.5,1,0.5},
	    		nOffsets = {0,-nB,0,-nT}
			 }
		})
	end
	tMyGrid[1][0]  = self.wndGrid:AddPixie({
  		strSprite = "RedFill",
		loc = {
		   	fPoints = {0.5,0,0.5,1},
	    	nOffsets = {-1,0,1,0}
		 }
	})
	tMyGrid[2][0]  = self.wndGrid:AddPixie({
  		strSprite = "RedFill",
		loc = {
		 	fPoints = {0,0.5,1,0.5},
	    	nOffsets = {0,-1,0,1}
		 }
	})
end
-----------------------------------------------------------------------------------------------
-- MovableFrames Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here3

-- on SlashCommand "/mf"
function MovableFrames:OnMovableFramesOn() 
	self.wndMain:Show(true) -- show the window
end

function MovableFrames:OnHud() 
	local oiself = Apollo.GetAddon("Interface")
	oiself.wndInterface:Show(true)
	oiself.wndInterface:FindChild("Content:General"):Show(false)
	oiself.wndInterface:FindChild("GeneralBtn"):SetCheck(false)
	oiself.wndInterface:FindChild("HudBtn"):SetCheck(true)
	oiself.wndInterface:FindChild("Content:HUD"):Show(true)
	oiself.wndInterface:FindChild("Content:WindowContent"):Show(false)
end

function MovableFrames:GetClass()
	local nClassID = GameLib.GetPlayerUnit():GetClassId()
	if nClassID == 1 then
		tAddonList[16][1] = "ClassResources"
		tAddonList[16][2] = "wndMain"
	elseif nClassID == 5 then
		tAddonList[16][1] = "ClassResources"
		tAddonList[16][2] = "wndMain"
	else
		tAddonList[16][1] = "ClassResources"
		tAddonList[16][2] = "wndMain"
	end
end

function MovableFrames:MoveFrame(frameToMove, bMove)
	local tAddon = Apollo.GetAddon(tAddonList[frameToMove][1])
	if tAddon then
		local MyWindow = self:getWindow(tAddon, tAddonList[frameToMove][2])
		if MyWindow then
			if bMove then
				local wnd = Apollo.LoadForm(self.xmlDoc, "BGmove", nil, self)
				local a1, a2, a3, a4 = MyWindow:GetAnchorPoints()
				local o1, o2, o3, o4 = MyWindow:GetAnchorOffsets()
				if frameToMove == 27 or frameToMove == 4 then
					wnd:SetAnchorPoints(a1, 1, a3, 1)
					wnd:SetAnchorOffsets(o1, o2-111, o3, o4-111)
				elseif frameToMove == 15 then
					local ABWindow = self:getWindow(Apollo.GetAddon(tAddonList[1][1]), tAddonList[1][2])
					local abo1, abo2, abo3, abo4 = ABWindow:GetAnchorOffsets()
					wnd:SetAnchorPoints(0.5, 1, 0.5, 1)
					wnd:SetAnchorOffsets(abo1+o1, o2-100, abo1+o3, o4-100)
				elseif frameToMove == 26 then
					local ABWindow = self:getWindow(Apollo.GetAddon(tAddonList[1][1]), tAddonList[1][2])
					local abo1, abo2, abo3, abo4 = ABWindow:GetAnchorOffsets()
					wnd:SetAnchorPoints(0.5, 1, 0.5, 1)
					wnd:SetAnchorOffsets(abo1+o1, o2-100, abo1+o3, o4-100)
					if not MyWindow:IsShown() then
						MyWindow:Show(true)
					end
					MyWindow:SetStyle("NewWindowDepth", false)
					MyWindow:SetStyle("CloseonExternalClick", false)
					MyWindow:SetStyle("NeverBringToFront", true)
				else
					wnd:SetAnchorPoints(MyWindow:GetAnchorPoints())
					wnd:SetAnchorOffsets(MyWindow:GetAnchorOffsets())
				end
				if not self.wndMain:FindChild("Options"):FindChild("Coords"):IsChecked() then
					wnd:FindChild("Locations"):Show(false)
				end

				if not self.wndMain:FindChild("Options"):FindChild("Pad"):IsChecked() then
					wnd:FindChild("Pad"):Show(false)
				end
				self:SetLoc(wnd)
				wnd:SetData(frameToMove)
				self.MovableWindows[frameToMove] = wnd
			end
			MyWindow:SetStyle("Moveable", bMove)
		end
	end
end

function MovableFrames:ShowFrame(frameToMove, bShow)
	local tAddon = Apollo.GetAddon(tAddonList[frameToMove][1])
	if tAddon then
		local MyWindow = self:getWindow(tAddon, tAddonList[frameToMove][2])
		if MyWindow then
			MyWindow:Show(bShow)
			if bShow then	
				tAddonList[frameToMove][6] = false
			else
				tAddonList[frameToMove][6] = true
			end
		end
	end
end

function MovableFrames:FramePositions(bGet)
	for i=1,nAddons do
		local tAddon = Apollo.GetAddon(tAddonList[i][1])
		if tAddon then
			local MyWindow = self:getWindow(tAddon, tAddonList[i][2])
			if MyWindow then
				if bGet then
					tAddonList[i][4] = {MyWindow:GetAnchorPoints()}
					tAddonList[i][5] = {MyWindow:GetAnchorOffsets()}
					if i == 5 then
						tAddonList[0][1] = false
						if MyWindow:IsShown() then
							tAddonList[0][1] = true
						end
					end
				else
					if tAddonList[i][4] then
						MyWindow:SetAnchorPoints(unpack(tAddonList[i][4]))
						MyWindow:SetAnchorOffsets(unpack(tAddonList[i][5]))
					end
				end
				MyWindow:SetStyle("Moveable", false)
			end
		end
	end
end

function MovableFrames:FrameHide()
	for i=1,nAddons do
		local tAddon = Apollo.GetAddon(tAddonList[i][1])
		if tAddon then
			local MyWindow = self:getWindow(tAddon, tAddonList[i][2])
			if MyWindow then
				if tAddonList[i][6] then
					MyWindow:Show(false)
					self.tItems[i]:FindChild("ShowHide"):SetCheck(true)
				else 
					if i == 15 and GameLib.GetDefaultRecallCommand() ~= nil and GameLib.HasBindPoint() then
						if not MyWindow:IsShown() then
							MyWindow:Show(true)
						end
					end
				end
			end
		end
	end
end

function MovableFrames:FrameScale()
	for i=1,nAddons do
		local tAddon = Apollo.GetAddon(tAddonList[i][1])
		if tAddon then
			local MyWindow = self:getWindow(tAddon, tAddonList[i][2])
			if MyWindow then
				if tAddonList[i][7] then
					MyWindow:SetScale(tAddonList[i][7])
				end
			end
		end
	end
end

function MovableFrames:SetLoc(wnd)
	local ao1, ao2, ao3, ao4 = wnd:GetAnchorOffsets()
	wnd:FindChild("Locations"):FindChild("Top"):SetText("Top: " .. ao2)
	wnd:FindChild("Locations"):FindChild("Left"):SetText("Left: " .. ao1)
	wnd:FindChild("Locations"):FindChild("Right"):SetText("Right: " .. ao3)
	wnd:FindChild("Locations"):FindChild("Bottom"):SetText("Bottom: " .. ao4)
end
-----------------------------------------------------------------------------------------------
-- MovableFramesForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function MovableFrames:OnOK()
	self.wndMain:Show(false) -- hide the window
end

-- when the Cancel button is clicked
function MovableFrames:OnCancel()
	self.wndMain:Show(false) -- hide the window
end

function MovableFrames:OnImport( wndHandler, wndControl )
	tAddonList = tAccountList[wndControl:GetSelectedData()]
	self:GetClass()
	self:FramePositions(false)	
end

function MovableFrames:OnSaveImport( wndHandler, wndControl, eMouseButton )
	bSave = true
	RequestReloadUI()
end

function MovableFrames:SortDown( wndHandler, wndControl, eMouseButton )
	questAlign = 0
	tAddonList[0][6] = 0
end

function MovableFrames:SortUp( wndHandler, wndControl, eMouseButton )
	questAlign = 2
	tAddonList[0][6] = 2
end

-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------
-- populate item list
function MovableFrames:PopulateItemList()
	self:DestroyItemList()
	for i = 1,nAddons-4 do
		if i == 8 then
			self:AddItem(i)
			self:AddItem(25)
			self:AddItem(28)
		elseif i == 15 then
			self:AddItem(i)
			self:AddItem(26)
		elseif i == 3 then
			self:AddItem(i)
			self:AddItem(27)
		else
			self:AddItem(i)
		end
	end
	self.wndItemList:ArrangeChildrenVert()
end
function MovableFrames:DestroyItemList()
	-- destroy all the wnd inside the list.
	for idx,wnd in ipairs(self.tItems) do
		wnd:Destroy()
	end

	-- clear the list item array.
	self.tItems = {}
	self.wndSelectedListItem = nil
end
-- add an item into the item list
function MovableFrames:AddItem(i)
	-- load the window item for the list item
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	-- keep track of the window item created
	self.tItems[i] = wnd

	-- give it a piece of data to refer to 
	local wndItemText = wnd:FindChild("Text")
	if wndItemText then -- make sure the text wnd exist
		wndItemText:SetText(tAddonList[i][3]) -- set the item wnd's text to "item i"
	end

	if i == 18 then
		wnd:FindChild("Button"):Show(false)
	end
	if i == 1 or i == 2 or i == 3 or i == 7 or i == 13 or i == 16 or i == 21 or i == 22 or i == 23 or i == 24 then
		wnd:FindChild("ShowHide"):Show(false)
	end
	wnd:SetData(i)
end

-- when a list item is selected
function MovableFrames:OnListItemSelected(wndHandler, wndControl)
    -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
      
	-- wndControl is the item selected - change its color to selected
	self.wndSelectedListItem = wndControl:GetParent()
	local frameToMove = self.wndSelectedListItem:GetData()
    self:MoveFrame(frameToMove, true)
end

function MovableFrames:OnShow(wndHandler, wndControl)
    -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
      
	-- wndControl is the item selected - change its color to selected
	self.wndListItem = wndControl:GetParent()
	local frameToShow = self.wndListItem:GetData()
    self:ShowFrame(frameToShow, true)
end
---------------------------------------------------------------------------------------------------
-- BGmove Functions
---------------------------------------------------------------------------------------------------
function MovableFrames:getWindow(tAddon, wnd)
	local MyWindow = nil
	if wnd == "wndMain" then
		MyWindow = tAddon.wndMain
	elseif wnd == "wndMenu" then
		MyWindow = tAddon.wndMenu
	elseif wnd == "wndAlertContainer" then
		MyWindow = tAddon.wndAlertContainer
	elseif wnd == "wndMinimized" then
		MyWindow = tAddon.wndMinimized
	elseif wnd == "g_wndDatachron" then
		MyWindow = g_wndDatachron
	elseif wnd == "wndHazardListForm" then
		MyWindow = tAddon.wndHazardListForm
	elseif wnd == "wndInvokeForm" then
		MyWindow = tAddon.wndInvokeForm
	elseif wnd == "wndLootStack" then
		MyWindow = tAddon.wndLootStack
	elseif wnd == "wndResourceBar" then
		MyWindow = tAddon.wndResourceBar
	elseif wnd == "wndCastFrame" then
		MyWindow = tAddon.wndCastFrame
	elseif wnd == "wndOppFrame" then
		MyWindow = tAddon.wndOppFrame	
	elseif wnd == "wndBar2" then
		MyWindow = tAddon.wndBar2
	elseif wnd == "wndBar3" then
		MyWindow = tAddon.wndBar3
	elseif wnd == "wndArt" then
		MyWindow = tAddon.wndArt
	elseif wnd == "wndMountFlyout" then
		MyWindow = tAddon.wndMountFlyout
	elseif wnd == "wndContainer" then
		MyWindow = tAddon.wndContainer
	elseif wnd == "luaUnitFrame" then
		MyWindow = tAddon["luaUnitFrame"].wndMainClusterFrame
	elseif wnd == "luaTargetFrame" then
		MyWindow = tAddon["luaTargetFrame"].wndMainClusterFrame
	elseif wnd == "luaFocusFrame" then
		MyWindow = tAddon["luaFocusFrame"].wndMainClusterFrame
	elseif wnd == "wndInteractForm" then
		MyWindow = Apollo.FindWindowByName("InteractForm")
	end
	return MyWindow
end

function MovableFrames:finalizeMove( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	if wndHandler ~= wndControl then
        return
    end
	local frameToMove = wndControl:GetData()
	local tAddon = Apollo.GetAddon(tAddonList[frameToMove][1])
	if tAddon then
		local MyWindow = self:getWindow(tAddon, tAddonList[frameToMove][2])
		if MyWindow then
			local wnd = self.MovableWindows[frameToMove]
			local a1, a2, a3, a4 = wnd:GetAnchorPoints()
			local o1, o2, o3, o4 = wnd:GetAnchorOffsets()
			if frameToMove == 27 or frameToMove == 4 then
				MyWindow:SetAnchorPoints(a1, 1, a3, 1)
				MyWindow:SetAnchorOffsets(o1, o2+111, o3, o4+111)
			elseif frameToMove == 15 then
				local ABWindow = self:getWindow(Apollo.GetAddon(tAddonList[1][1]), tAddonList[1][2])
				local abo1, abo2, abo3, abo4 = ABWindow:GetAnchorOffsets()
				MyWindow:SetAnchorPoints(0.5, 1, 0.5, 1)
				MyWindow:SetAnchorOffsets(o1-abo1, o2+100, o3-abo1, o4+100)
			elseif frameToMove == 26 then
				local ABWindow = self:getWindow(Apollo.GetAddon(tAddonList[1][1]), tAddonList[1][2])
				local abo1, abo2, abo3, abo4 = ABWindow:GetAnchorOffsets()
				MyWindow:SetAnchorPoints(0.5, 1, 0.5, 1)
				MyWindow:SetAnchorOffsets(o1-abo1, o2+100, o3-abo1, o4+100)
			else
				MyWindow:SetAnchorPoints(wnd:GetAnchorPoints())
				MyWindow:SetAnchorOffsets(wnd:GetAnchorOffsets())
			end
		end
	end
end

function MovableFrames:OnChangeLoc( wndHandler, wndControl, x, y, wndSource, strType, iData, bDragDropHasBeenReset )
	if wndHandler ~= wndControl then
        return
    end
	self:SetLoc(wndHandler)
end

function MovableFrames:OnMoveWindow( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then
        return
    end
	local MyWindow = wndHandler:GetParent():GetParent()
	local ao1, ao2, ao3, ao4 = MyWindow:GetAnchorOffsets()
	local buttonName = wndHandler:GetName()
	if buttonName == "Up" then
		ao2 = ao2 - 1
		ao4 = ao4 - 1
	elseif buttonName == "Down" then
		ao2 = ao2 + 1
		ao4 = ao4 + 1
	elseif buttonName == "Left" then
		ao1 = ao1 - 1
		ao3 = ao3 - 1
	elseif buttonName == "Right" then
		ao1 = ao1 + 1
		ao3 = ao3 + 1
	end
	MyWindow:SetAnchorOffsets(ao1, ao2, ao3, ao4)
	self:SetLoc(MyWindow)
end

---------------------------------------------------------------------------------------------------
-- ListItem Functions
---------------------------------------------------------------------------------------------------

function MovableFrames:DestroyMoveWindow( wndHandler, wndControl, eMouseButton )
	local f2m = wndControl:GetParent():GetData()
	if f2m then
		if self.MovableWindows[f2m] then
			self.MovableWindows[f2m]:Destroy()
		end
		self:MoveFrame(f2m, false)
		
		if f2m == 26 then
			local tAddon = Apollo.GetAddon(tAddonList[f2m][1])
			local MyWindow = self:getWindow(tAddon, tAddonList[f2m][2])
			MyWindow:Show(false)
			MyWindow:SetStyle("NewWindowDepth", true)
			MyWindow:SetStyle("CloseonExternalClick", true)
			MyWindow:SetStyle("NeverBringToFront", false)
		end
	end
end

function MovableFrames:OnHide( wndHandler, wndControl, eMouseButton )
    -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
      
	-- wndControl is the item selected - change its color to selected
	self.wndListItem = wndControl:GetParent()
	local frameToShow = self.wndListItem:GetData()
    self:ShowFrame(frameToShow, false)
end

---------------------------------------------------------------------------------------------------
-- Scale Functions
---------------------------------------------------------------------------------------------------
function MovableFrames:OnShowScale( wndHandler, wndControl, eMouseButton )
   -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
    local wnd = Apollo.LoadForm(self.xmlDoc, "Scale", nil, self)
	-- wndControl is the item selected - change its color to selected
	
	local f2s = wndControl:GetParent():GetData()
	wnd:FindChild("Addon"):SetText(tAddonList[f2s][3])
	if tAddonList[f2s][7] then
		wnd:FindChild("AoS"):SetText(tAddonList[f2s][7])
	end
	wnd:SetData(f2s)
	self.ScaleWindows[f2s] = wnd
end

function MovableFrames:OnHideScale( wndHandler, wndControl, eMouseButton )
	if wndHandler ~= wndControl then
        return
    end
	local f2s = wndControl:GetParent():GetData()
	self.ScaleWindows[f2s]:Show(false)
	self.ScaleWindows[f2s]:Destroy()
end

function MovableFrames:SetScale( wndHandler, wndControl, fNewValue, fOldValue )
	if wndHandler ~= wndControl then
        return
    end
	local f2s = wndControl:GetParent():GetData()
	local fScale = math.floor(fNewValue * 20) * 0.05
	wndControl:GetParent():FindChild("AoS"):SetText(fScale)
	local tAddon = Apollo.GetAddon(tAddonList[f2s][1])
	local MyWindow = self:getWindow(tAddon, tAddonList[f2s][2])
	if MyWindow then
		MyWindow:SetScale(fScale)
		tAddonList[f2s][7] = fScale
	end
end

-----------
-- Hooks
-----------
local XPBar = Apollo.GetAddon("XPBar")
if XPBar then
	local origRedrawAllPastCooldown = XPBar.RedrawAllPastCooldown
	XPBar.RedrawAllPastCooldown = function(self)
		origRedrawAllPastCooldown(self)
		if tAddonList[20][6] then
			self.wndArt:Show(false)
		end
	end
end

local ActionBarFrame = Apollo.GetAddon("ActionBarFrame")
if ActionBarFrame then
	local origRedrawBarVisibility = ActionBarFrame.RedrawBarVisibility
	ActionBarFrame.RedrawBarVisibility = function(self)
		origRedrawBarVisibility(self)
		if tAddonList[19][6] then
			self.wndArt:Show(false)
		end
	end

	local origInitializeBars = ActionBarFrame.InitializeBars
	ActionBarFrame.InitializeBars = function(self)
		local MF = Apollo.GetAddon("MovableFrames")
		if self.wndBar2:GetData() ~= "MF" then
			self.wndBar2:Show(false)
			self.wndBar2:Destroy()
			self.wndBar2 = Apollo.LoadForm(MF.xmlDoc, "Bar2ButtonContainer", "FixedHudStratum", self)
			self.wndBar2:SetData("MF")
		end
		if self.wndBar3:GetData() ~= "MF" then
			self.wndBar3:Show(false)
			self.wndBar3:Destroy()
			self.wndBar3 = Apollo.LoadForm(MF.xmlDoc, "Bar3ButtonContainer", "FixedHudStratum", self)
			self.wndBar3:SetData("MF")
		end
		origInitializeBars(self)
	end
end

local RecallFrame = Apollo.GetAddon("RecallFrame")
if RecallFrame then
	--RecallFrame.OnLoad = function(self)
		--self.xmlDoc = XmlDoc.CreateFromFile("RecallFrame.xml")
		--self.xmlDoc:RegisterCallback("OnDocumentReady", self)
	--end

	--origRefreshDefaultCommand = RecallFrame.RefreshDefaultCommand
	RecallFrame.RefreshDefaultCommand = function(self)
		--origRefreshDefaultCommand(self)
		--if tAddonList[15][6] then
		--	self.wndMain:Show(false)
		--else
		--	if not self.wndMain:IsShown() then		
		--		self.wndMain:Show(true)
		--	end
		--end
	end
end

local HealthShieldBar = Apollo.GetAddon("HealthShieldBar")
if HealthShieldBar then
	origOnFrameUpdate = HealthShieldBar.OnFrameUpdate
	HealthShieldBar.OnFrameUpdate = function(self)
		origOnFrameUpdate(self)
		if tAddonList[4][6] then
			self.wndMain:Show(false)
		else
			self.wndMain:Show(true)
		end
	end
end

local CastBar = Apollo.GetAddon("CastBar")
if CastBar then
CastBar.OnUpdate= function(self)
	local unitPlayer = GameLib.GetPlayerUnit()
	local nRectLeft, nRectTop, nRectRight, nRectBottom = self.wndCastFrame:GetRect()
	if not unitPlayer then
		return
	end
	if self.tCurrentOpSpell ~= nil then
		if self.tCurrentOpSpell.eCastMethod == Spell.CodeEnumCastMethod.RapidTap then
			self:DrawSingleBarFrameCircle(self.wndOppBarCircle)
		elseif self.tCurrentOpSpell.eCastMethod == Spell.CodeEnumCastMethod.PressHold then
			self:DrawSingleBarFrame(self.wndOppBar)
		elseif self.tCurrentOpSpell.eCastMethod == Spell.CodeEnumCastMethod.ChargeRelease then
			self:DrawSingleBarFrameTiered(self.wndOppBarTiered)
		end
	end
	local bShowSimpleCast = true
	if tAddonList[27][6] then
		bShowSimpleCast = false
	end
	local bShowCasting = false
	local bEnableGlow = false
	local nZone = 0
	local nMaxZone = 0
	local fDuration = 0
	local fElapsed = 0
	local strSpellName = ""
	local nElapsed = 0
	local eType = Unit.CodeEnumCastBarType.None
	if unitPlayer:ShouldShowCastBar() then
		if bShowSimpleCast then
			self.bIsShown = true
			eType = unitPlayer:GetCastBarType()
			if eType == Unit.CodeEnumCastBarType.Normal then
				self.wndCastFrame:FindChild("CastingProgress"):SetFullSprite("SpellChargeFull")
				bShowCasting = true
				bEnableGlow = true
				nZone = 0
				nMaxZone = 1
				fDuration = unitPlayer:GetCastDuration()
				fElapsed = unitPlayer:GetCastElapsed()
				self.wndCastFrame:FindChild("CastingProgress"):SetTickLocations(0, 100, 200, 300)
				strSpellName = unitPlayer:GetCastName()
			end
			Apollo.SetGlobalAnchor("CenterTextBottom", 0.0, nRectTop, true)
		end
	else
		self.bIsShown = false
		self.wndCastFrame:Show(false)
		Apollo.SetGlobalAnchor("CenterTextBottom", 0.0, nRectBottom, true)
	end
	if bShowCasting and fDuration > 0 and nMaxZone > 0 then
		self.wndCastFrame:Show(bShowCasting)
		self.wndCastFrame:FindChild("CastingProgress"):SetMax(fDuration)
		self.wndCastFrame:FindChild("CastingProgress"):SetProgress(fElapsed)
		self.wndCastFrame:FindChild("CastingProgress"):EnableGlow(bEnableGlow)
		self.wndCastFrame:FindChild("CastingProgressText"):SetText(strSpellName)
	end

	end
end



-----------------------------------------------------------------------------------------------
-- MovableFrames Instance
-----------------------------------------------------------------------------------------------
local MovableFramesInst = MovableFrames:new()
MovableFramesInst:Init()
